<?php
/*
Plugin Name: OW Redirects
Plugin URI: https://olabaworks.com/
Description: Manage 301 Redirects efficiently. No PRO or Premium annoyances. Prevents repeated redirects. No database tables used, just JSON files. Management using a fast and lightweight data-tables grid.
Author: Olabaworks
Author URI: https://olabaworks.com/
Text Domain: ow-redirects
Domain Path: /languages/
Version: 1.1.0
*/

class owRedirects {

	protected $PLUGIN_FILE;
	protected $PLUGIN_DIR;
	protected $PLUGIN_URL;

	protected $ADMIN_PAGE_TITLE;
	protected $ADMIN_MENU_SLUG;

	protected $JSON_FILE_PATH;

	public function __construct() {

        $aUploadDir = wp_upload_dir();

		$this->PLUGIN_FILE      = plugin_basename( __FILE__ );
		$this->PLUGIN_DIR       = untrailingslashit( dirname( __FILE__ ) );
		$this->PLUGIN_URL       = plugins_url() . '/' . ( basename( dirname( __FILE__ ) ) );
		$this->ADMIN_PAGE_TITLE = 'OW Redirects';
		$this->ADMIN_MENU_SLUG  = 'tools-ow-redirects';
		$this->JSON_FILE_PATH   =  $aUploadDir['basedir'] . '/json/redirects.json';

		add_action( 'admin_notices', function () {
			$this->adminPageController();
		}, 99 );

		add_action( 'admin_menu', function () {
			add_management_page( $this->ADMIN_PAGE_TITLE, $this->ADMIN_PAGE_TITLE, 'administrator', $this->ADMIN_MENU_SLUG, function () {
				$this->adminPageView();
			}, 3 );
		} );

		add_action( 'wp', [ $this, 'doRedirects' ], 99 );
	}

	public function doRedirects() {
		$url       = $this->getUrl();
		$redirects = $this->getRedirects( 'search' );
		if ( ! empty( $redirects ) &&
		     in_array( $url, array_keys( $redirects ) ) ||
		     in_array( rtrim( $url, '/' ), array_keys( $redirects ) ) ) {
			header( 'Cache-Control: no-store, no-cache, must-revalidate, max-age=0' );
			header( 'Cache-Control: post-check=0, pre-check=0', false );
			header( 'Pragma: no-cache' );
			header( 'Location: ' . $redirects[ $url ], true, 301 );
			exit();
		}
	}

	private function adminPageController() {

		if ( empty( $_GET['page'] ) OR ! strstr( $_GET['page'], $this->ADMIN_MENU_SLUG ) ) {
			return;
		}

		$action = ! empty( $_GET['action'] ) ? esc_attr( $_GET['action'] ) : ( ! empty( $_POST['action'] ) ? esc_attr( $_POST['action'] ) : '' );

		if ( empty( $action ) OR (
				$action !== 'delete' && $action !== 'backup' && $action !== 'add' ) ) {
			return;
		}

		$redirects = $this->getRedirects();

		if ( empty( $redirects ) && $action !== 'add' ) {
			return;
		}

		$msg_class = 'success';
		$msg       = __( 'Cambios guardados.' );

		switch ( $action ) :

			case 'backup' :

				$bck_file_path = substr( $this->JSON_FILE_PATH, 0, - 5 ) . '-bck-' . date( 'YmdHi' ) . '.json';
				@file_put_contents( $bck_file_path, json_encode( $redirects ) );
				$msg = __( 'Se ha guardado una copia en formato JSON en:' ) . '<br>' . $bck_file_path;
				break;

			case 'add' :

				$source_url = ! empty( $_POST['source_url'] ) ? filter_var( esc_attr( $_POST['source_url'] ), FILTER_VALIDATE_URL ) : '';
				$target_url = ! empty( $_POST['target_url'] ) ? filter_var( esc_attr( $_POST['target_url'] ), FILTER_VALIDATE_URL ) : '';

				if ( ! empty( $source_url ) && ! empty( $target_url ) ) {

					$source_url = ! strstr( $source_url, '?' ) ? ( rtrim( $source_url, '/' ) . '/' ) : $source_url;
					$target_url = ! strstr( $target_url, '?' ) ? ( rtrim( $target_url, '/' ) . '/' ) : $target_url;

					$redirects_search = $this->getRedirects( 'search' );
					if ( in_array( $source_url, array_keys( $redirects_search ) ) ) {
						$msg       = __( 'Ya existe una redirección para la URL de origen.' );
						$msg       .= '<br><br>DE:<br>' . $source_url . '<br>A:<br>' . $redirects_search[ $source_url ];
						$msg_class = 'error';
					} else {
						if ( $source_url === $target_url ) {
							$msg       = __( 'La URL de origen no puede ser igual que la de destino.' );
							$msg_class = 'error';
						} else {
							$redirects[] = [
								'source_url' => $source_url,
								'target_url' => $target_url
							];
						}
					}

				} else {
					$msg       = __( 'URL origen o destino no válida.' );
					$msg_class = 'error';
				}

				break;

			case 'delete' :

				$id = ! empty( $_GET['id'] ) ? intval( esc_attr( $_GET['id'] ) ) : 0;
				if ( ! empty( $id ) && ! empty( $redirects[ ( $id - 1 ) ] ) ) {
					unset( $redirects[ ( $id - 1 ) ] );
				}
				break;

		endswitch;

		file_put_contents( $this->JSON_FILE_PATH, json_encode( $redirects ) );

		if ( ! empty( $msg ) ) {
			echo '<div class="notice notice-' . $msg_class . ' is-dismissible">';
			echo '<p>';
			echo $msg;
			echo '</p>';
			echo '</div>';
		}
	}

	private function adminPageView() {
		echo '<style>';
		echo 'table.dataTable thead .sorting{background:url("' . $this->PLUGIN_URL . '/admin-page/assets/css/datatables/images/sort_both.png") no-repeat center right}';
		echo 'table.dataTable thead .sorting_asc{background:url("' . $this->PLUGIN_URL . '/admin-page/assets/css/datatables/images/sort_asc.png") no-repeat center right}';
		echo 'table.dataTable thead .sorting_desc{background:url("' . $this->PLUGIN_URL . '/admin-page/assets/css/datatables/images/sort_desc.png") no-repeat center right}';
		echo 'table.dataTable thead .sorting_asc_disabled{background:url("' . $this->PLUGIN_URL . '/admin-page/assets/css/datatables/images/sort_asc_disabled.png") no-repeat center right}';
		echo 'table.dataTable thead .sorting_desc_disabled{background:url("' . $this->PLUGIN_URL . '/admin-page/assets/css/datatables/images/sort_desc_disabled.png") no-repeat center right}';
		echo file_get_contents( $this->PLUGIN_DIR . '/admin-page/assets/css/datatables/jquery.dataTables.min.css' );
		echo file_get_contents( $this->PLUGIN_DIR . '/admin-page/assets/css/ow-dataTables.css' );
		echo file_get_contents( $this->PLUGIN_DIR . '/admin-page/assets/css/ow-modal.css' );
		echo file_get_contents( $this->PLUGIN_DIR . '/admin-page/assets/css/ow-admin-page.css' );
		echo '</style>';
		echo '<script>';
		echo file_get_contents( $this->PLUGIN_DIR . '/admin-page/assets/js/jquery.dataTables.min.js' );
		echo file_get_contents( $this->PLUGIN_DIR . '/admin-page/assets/js/ow-dataTables.js' );
		echo '</script>';
		echo '<script>';
		echo 'var ow_redirects_rows = ' . json_encode( $this->getRedirects( 'display' ) ) . ';';
		echo 'window.addEventListener(\'load\',function(){initDataTables();});';
		echo '</script>';
		include( $this->PLUGIN_DIR . '/admin-page/view.php' );
	}

	private function getRedirects( $mode = 'default' ) {

		$file_data = json_decode( file_get_contents( $this->JSON_FILE_PATH ), JSON_OBJECT_AS_ARRAY );
		if ( empty( $file_data ) ) {
			return [];
		}

		$i                 = 0;
		$redirects         = [];
		$redirects_display = [];
		$redirects_search  = [];

		foreach ( $file_data as $f ) {

			// Exclude repeated
			if ( in_array( $f['source_url'], array_keys( $redirects_search ) ) ) {
				continue;
			}

			$redirects[]                          = $f;
			$redirects_search[ $f['source_url'] ] = $f['target_url'];
			if ( $mode === 'display' ) {
				$i ++;
				$redirects_display[] = [
					'id'         => $i,
					'source_url' => '<a href="' . $f['source_url'] . '" target="_blank">' . $f['source_url'] . '</a>',
					'target_url' => '<a href="' . $f['target_url'] . '" target="_blank">' . $f['target_url'] . '</a>',
					'actions'    => '<a onclick="if (!confirm(\'¿Eliminar este registro?\')) { event.preventDefault(); }" href="' . admin_url( 'tools.php' ) . '?page=' . $this->ADMIN_MENU_SLUG . '&amp;action=delete&amp;id=' . $i . '" class="button" title="' . __( 'Eliminar' ) . '">X</a>'
				];
			}
		}

		return ( $mode === 'display' ) ? $redirects_display : ( $mode === 'search' ? $redirects_search : $redirects );
	}

	private function getUrl() {
		return @( $_SERVER["HTTPS"] != 'on' ) ? 'http://' . $_SERVER["HTTP_HOST"] . $_SERVER["REQUEST_URI"] : 'https://' . $_SERVER["HTTP_HOST"] . $_SERVER["REQUEST_URI"];
	}
}

new owRedirects();
